"use strict";

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var __importDefault = undefined && undefined.__importDefault || function (mod) {
    return mod && mod.__esModule ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var index_alipay_1 = __importDefault(require("../mtop/index.alipay"));
var tg_iot_interface_api_device_panel_1 = require("@alife/tg-iot-interface-api-device-panel");
var SkillIdMissingError = {
    code: 'internal_error',
    msg: 'skill_id 为空'
};
var TMPSDK = /** @class */function () {
    function TMPSDK() {
        this.productInfoCache = {};
    }
    TMPSDK.prototype.controlDevice = function (request) {
        var productKey = request.productKey;
        var deviceId = request.deviceId;
        var method = request.method;
        var params = request.params;
        var success = request.success;
        var fail = request.fail;
        var complete = request.complete;
        var skillId = request.skillId;
        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }
        if (!method || tg_iot_interface_api_device_panel_1.EMTHOD_ATTRIBUTE_SET !== method && tg_iot_interface_api_device_panel_1.EMTHOD_SERVICE_INVOKE !== method) {
            throw new Error('method is invalid');
        }
        if (!params) {
            throw new Error('params is invalid');
        }
        this._handleSkillId(productKey, skillId, function (skillId) {
            // 非 515 走猫精接口
            if (515 != skillId) {
                index_alipay_1.default.sendMtop({
                    api: 'mtop.alibaba.aicloud.iot.deviceControlForAlipay',
                    v: '1.0',
                    needLogin: true,
                    data: {
                        devId: deviceId,
                        skillId: skillId,
                        params: (0, _stringify2.default)({
                            method: method,
                            params: params,
                            devId: deviceId,
                            "msgId": "",
                            "version": "1.0"
                        })
                    },
                    success: success,
                    fail: fail,
                    complete: complete
                });
                return;
            }
            // 515 为纯飞燕设备
            // 走弱生态链路， 接口文档参考:https://yuque.antfin-inc.com/ailabsfe/clpsv1/vaxx2g
            if (method === tg_iot_interface_api_device_panel_1.EMTHOD_ATTRIBUTE_SET) {
                index_alipay_1.default.sendMtop({
                    api: 'mtop.alibaba.aicloud.iot.aliyunH5ModuleForAlipay',
                    v: '1.0',
                    needLogin: true,
                    data: {
                        func: 'controlByApp',
                        args: (0, _stringify2.default)([deviceId, skillId, (0, _stringify2.default)(params)])
                    },
                    success: success,
                    fail: fail,
                    complete: complete
                });
            } else if (method === tg_iot_interface_api_device_panel_1.EMTHOD_SERVICE_INVOKE) {
                index_alipay_1.default.sendMtop({
                    api: 'mtop.alibaba.aicloud.iot.aliyunH5ModuleForAlipay',
                    v: '1.0',
                    needLogin: true,
                    data: {
                        func: 'serviceInvoke',
                        args: (0, _stringify2.default)([{
                            deviceId: deviceId,
                            skillId: skillId,
                            // @ts-ignore
                            identifier: params.identifier || '',
                            // @ts-ignore
                            args: (0, _stringify2.default)(params.args || {})
                        }])
                    },
                    success: success,
                    fail: fail,
                    complete: complete
                });
            }
        }, function (err) {
            fail && fail(err);
            complete && complete(err);
        })();
    };
    TMPSDK.prototype.controlMeshDevice = function (request) {
        throw new Error('Not Implemetated !');
    };
    TMPSDK.prototype.getDeviceConnectStatus = function (request) {
        throw new Error('Not Implemetated !');
    };
    TMPSDK.prototype.getDeviceStatus = function (request) {
        var productKey = request.productKey;
        var deviceId = request.deviceId;
        var success = request.success;
        var fail = request.fail;
        var complete = request.complete;
        var skillId = request.skillId;
        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }
        this._handleSkillId(productKey, skillId, function (skillId) {
            index_alipay_1.default.sendMtop({
                api: 'mtop.alibaba.aicloud.iot.getDeviceStatusForAlipay',
                v: '1.0',
                needLogin: true,
                data: {
                    devId: deviceId,
                    skillId: skillId
                },
                success: success,
                fail: fail,
                complete: complete
            });
        }, function (err) {
            fail && fail(err);
            complete && complete(err);
        })();
    };
    TMPSDK.prototype.getProductInfo = function (request) {
        var productKey = request.productKey;
        var skillId = request.skillId;
        var deviceId = request.deviceId;
        var success = request.success;
        var fail = request.fail;
        var complete = request.complete;
        if (!productKey) {
            throw new Error('productKey is invalid');
        }
        if (skillId !== 515) {
            index_alipay_1.default.sendMtop({
                api: 'mtop.alibaba.aicloud.iotopenplatformapp.getProductConfig',
                v: '1.0',
                data: {
                    productKey: productKey
                },
                success: success,
                fail: fail,
                complete: complete
            });
            return;
        }
        // skillId 为 515 时
        // 走弱生态链路， 接口文档参考:https://yuque.antfin-inc.com/ailabsfe/clpsv1/vaxx2g
        index_alipay_1.default.sendMtop({
            api: 'mtop.alibaba.aicloud.iot.aliyunH5ModuleForAlipay',
            v: '1.0',
            needLogin: true,
            data: {
                func: 'queryProductInfo',
                args: (0, _stringify2.default)([{
                    deviceId: deviceId,
                    skillId: skillId.toString()
                }])
            },
            success: success,
            fail: fail,
            complete: complete
        });
    };
    TMPSDK.prototype.getProductTSL = function (request) {
        var productKey = request.productKey;
        var deviceId = request.deviceId;
        var success = request.success;
        var fail = request.fail;
        var complete = request.complete;
        var skillId = request.skillId;
        if (!productKey) {
            throw new Error('productKey is invalid');
        }
        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }
        this._handleSkillId(productKey, skillId, function (skillId) {
            index_alipay_1.default.sendMtop({
                api: 'mtop.alibaba.aicloud.iot.getDeviceInfoForAlipay',
                v: '1.0',
                needLogin: true,
                data: {
                    productKey: productKey,
                    devId: deviceId,
                    skillId: skillId
                },
                success: success,
                fail: fail,
                complete: complete
            });
        }, function (err) {
            fail && fail(err);
            complete && complete(err);
        })();
    };
    TMPSDK.prototype.deleteDevice = function (request) {
        var productKey = request.productKey;
        var deviceId = request.deviceId;
        var success = request.success;
        var fail = request.fail;
        var complete = request.complete;
        var skillId = request.skillId;
        if (!productKey) {
            throw new Error('productKey is invalid');
        }
        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }
        this._handleSkillId(productKey, skillId, function (skillId) {
            index_alipay_1.default.sendMtop({
                api: 'mtop.alibaba.aicloud.iot.deletedevice',
                v: '1.0',
                needLogin: true,
                data: {
                    unbindRequestJSONString: (0, _stringify2.default)({
                        productKey: productKey,
                        devId: deviceId,
                        skillId: skillId
                    }),
                    deviceInfo: (0, _stringify2.default)({
                        "botId": 10
                    })
                },
                success: success,
                fail: fail,
                complete: complete
            });
        }, function (err) {
            fail && fail(err);
            complete && complete(err);
        })();
    };
    TMPSDK.prototype.log = function (msg) {
        console.log(msg);
    };
    /**
     * skill id 缺失时的补偿逻辑
     * @param deviceId 设备id
     * @param skillId 技能id
     * @param success 成功回调
     * @param fail 失败回调
     */
    TMPSDK.prototype._handleSkillId = function (productKey, skillId, _success, _fail) {
        var _this = this;
        return function () {
            // 如果 skillId 未设置，尝试从缓存获取
            if (!skillId && _this.productInfoCache[productKey]) {
                skillId = _this.productInfoCache[productKey].skillId;
            }
            // 有 skillId, 触发成功回调
            if (skillId) {
                _success && _success(skillId);
                return;
            }
            if (!productKey) {
                _fail && _fail(new Error('productKey is invalid'));
            }
            // 拉取设备列表，以获取 skillId
            _this.getProductInfo({
                productKey: productKey,
                success: function success(res) {
                    _this.productInfoCache[productKey] = res;
                    skillId = res.skillId;
                    _success && _success(skillId);
                },
                fail: function fail(err) {
                    _fail && _fail(err);
                }
            });
        };
    };
    /**
     * 空实现，等子类覆盖
     * @param listener
     */
    TMPSDK.prototype.onDeviceStatusChange = function (listener) {};
    /**
     * 空实现，等子类覆盖
     * @param listener
     */
    TMPSDK.prototype.offDeviceStatusChange = function (listener) {};
    return TMPSDK;
}();
exports.TMPSDK = TMPSDK;
;
exports.default = new TMPSDK();