"use strict";

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

Object.defineProperty(exports, "__esModule", { value: true });
var defaultRequest = {
    api: '',
    v: '1.0',
    data: {},
    needLogin: false,
    success: function success(ret) {
        console.warn('unhandled response:', (0, _stringify2.default)(ret));
    }
};
var MTop = /** @class */function () {
    function MTop() {
        this._defaultHeaders = {};
        this._debug = true;
    }
    MTop.prototype.setEnv = function (env) {
        if (!env) {
            // '' 代表线上
            env = '';
        }
        (0, _assign2.default)(this._defaultHeaders, {
            'EagleEye-UserData': "scm_project=" + env
        });
    };
    MTop.prototype.setDebug = function (debug) {
        this._debug = debug;
    };
    MTop.prototype.sendMtop = function (request) {
        var _this = this;
        // 添加默认值
        request = (0, _assign2.default)({}, defaultRequest, request);
        var api = request.api;
        var v = request.v;
        var needLogin = request.needLogin || false;
        var data = request.data;
        var success = request.success;
        var fail = request.fail;
        var complete = request.complete;
        if (!api) {
            throw new Error('api is invalid');
        }
        if (!v) {
            throw new Error('v is invalid');
        }
        // my.sendMtop 接口存在 bug,
        // 发送需要登录的 MTop 请求时，登录态处理会出错，使用 mtop 接口代替
        // my.sendMtop({
        //     api,
        //     v,
        //     needLogin,
        //     data,
        //     success,
        //     fail,
        //     complete,
        // });
        try {
            if (this._debug) {
                console.log('>> before send mtop', api, v, needLogin, data);
            }
            my.call('mtop', {
                apiName: api,
                apiVersion: v,
                needEcodeSign: needLogin,
                useGenieEnv: needLogin,
                headers: this._defaultHeaders,
                data: data
            }, function (ret) {
                if (_this._debug) {
                    console.log('>> after send mtop', (0, _stringify2.default)(ret));
                }
                var data = ret.data,
                    internalRet = ret.ret,
                    api = ret.api;
                if (!data) {
                    var invalidResponseError = {
                        code: 'invalid response',
                        msg: '无效的返回数据'
                    };
                    fail && fail(invalidResponseError);
                    complete && complete(invalidResponseError);
                    return;
                }
                // 标准的 MTOP 处理逻辑
                var isSuccess = data.success,
                    model = data.model,
                    retValue = data.retValue,
                    msgInfo = data.msgInfo,
                    msgCode = data.msgCode;
                // isSuccess maybe true, false, 'true', 'false'
                // mtop get from def MTop Trigger use retValue instead of model
                if (isSuccess == 'true' || isSuccess === true) {
                    success(model || retValue);
                    complete && complete(model || retValue);
                } else {
                    var responseError = {
                        code: msgCode,
                        msg: msgInfo
                    };
                    // 从 ret 中获取 errorCode
                    if (!responseError.code) {
                        var retMessage = internalRet[0];
                        var messageArr = retMessage.split('::');
                        (0, _assign2.default)(responseError, {
                            code: messageArr[0],
                            msg: messageArr[1]
                        });
                    }
                    console.warn('>> response error happens when send mtop', (0, _stringify2.default)(responseError));
                    fail && fail(responseError);
                    complete && complete(responseError);
                }
                ;
            });
        } catch (err) {
            console.warn('>> exception happens when send mtop', (0, _stringify2.default)(err));
            fail && fail(err);
            complete && complete(err);
        }
    };
    return MTop;
}();
exports.MTop = MTop;
exports.default = new MTop();