import IAddDeviceAPI, { StartAddDeviceOptions, StopAddDeviceOptions, AddDeviceStatus } from '@alife/tg-iot-interface-api-add-device';
// import DeviceSmartConfig from '@alife/iot-mini-program-device-smart-config';
import prepare from './prepare';
import bind from './bind';

class AddDevice implements IAddDeviceAPI {

    // private _deviceSmartConfigClient: DeviceSmartConfig;

    private _currentStatus: AddDeviceStatus;

    private _listenerManager: Array<(AddDeviceStatus)=>void>;

    get currentAddDeviceStatus() : AddDeviceStatus {
        return this._currentStatus;
    }

    constructor(){
        this._listenerManager = new Array<(AddDeviceStatus)=>void>();
    }

    /**
     * 开始添加设备
     * @param options 开始添加设备的信息描述
     */
    public startAddDevice(options: StartAddDeviceOptions) {
        const { productKey, ssid, fail, complete } = options;

        if (!productKey) {
            throw new Error('productKey is missing');
        }

        if (!ssid) {
            throw new Error('ssid is missing');
        }

        if(this._currentStatus && this._currentStatus.status === '')

        this._currentStatus = {
            status: 'prechecking',
        };
        this._notifyListeners();

        prepare.getDeviceConfigToken({
            productKey,
            success: (res) => {
                this._doProvision(options, res);
            },
            fail, 
            complete,
        });
    }

    /**
     * 停止添加设备
     * @param options 停止添加设备的信息描述
     */
    public stopAddDevice(options: StopAddDeviceOptions) {
        // if (!this._deviceSmartConfigClient) {
            // return;
        // }

        // this._deviceSmartConfigClient.stop();
        // this._deviceSmartConfigClient = null;
    }

    /**
     * 开始监听设备添加过程变化
     * @param listener 监听器
     */
    public onAddDeviceStatusChagne(listener: (status: AddDeviceStatus) => void) {
        if (this._listenerManager.indexOf(listener) !== -1) {
            console.warn('duplicated listener');
            return;
        }

        this._listenerManager.push(listener);
    }

    /**
     * 停止监听设备添加过程变化
     * @param listener 需要停止监听的监听器; 若为空，则停止所有监听器
     */
    public offAddDeviceStatusChange(listener?: (status: AddDeviceStatus) => void) {
        if (!listener) {
            this._listenerManager.splice(0, this._listenerManager.length);
            return;
        }

        const index = this._listenerManager.indexOf(listener);
        if (index === -1) {
            console.warn('invalid listener');
            return;
        }

        this._listenerManager.splice(index, 1);
    }

    /**
     * 进行配网
     * @param productKey 待配网的产品
     * @param token 产品的配网秘钥
     * @param success 成功回调
     * @param fail 失败回调
     * @param complte 完成回调
     */
    private _doProvision(options: StartAddDeviceOptions, token:string) {
        this._currentStatus = {
            status: 'provisioning',
        };
        this._notifyListeners();

        const { productKey, ssid, pw, timeout } = options;
        console.log('>> _doProvision', productKey, ssid, pw, token, timeout);
        
        // this._deviceSmartConfigClient = new DeviceSmartConfig({
        //     productKey,
        //     securityKey: token,
        //     ssid,
        //     password:pw,
        //     timeout,
        //     onProgress:(options) => {
        //         console.log('>> onProgress', JSON.stringify(options));
        //         const { status, message } = options;
        //         if (status === 'fail') {
        //             this._currentStatus = {
        //                 status: 'fail',
        //                 extraData: {
        //                     code: 'timeout',
        //                     msg: message,
        //                 }
        //             }
        //             this._notifyListeners();
        //             this._deviceSmartConfigClient.stop();
        //             this._deviceSmartConfigClient = null;
        //             return;
        //         }
          
        //         if (status === 'success') {
        //             const {productKey, deviceName, token} = message;
        //             this._doBind(productKey, deviceName, token);
        //             this._deviceSmartConfigClient.stop();
        //             this._deviceSmartConfigClient = null;
        //             return;
        //         }
        //     }
        // });
        // this._deviceSmartConfigClient.start().then(()=>{
        //     console.log('>> _deviceSmartConfigClient.start', 'success');
        //     options.success({});
        // }).catch((err)=>{
        //     console.log('>> _deviceSmartConfigClient.start', 'failed', err);
        //     options.fail && options.fail(err);
        // });
    }

    /**
     * 绑定设备
     * @param productKey 待绑定的产品
     * @param deviceName 待绑定的设备名称
     * @param token 待绑定的 token
     */
    private _doBind(productKey:string, deviceName:string, token:string) {
        this._currentStatus = {
            status: 'binding',
            extraData: {
                productKey,
                deviceName,
                token,
            },
        };
        this._notifyListeners();

        bind.bindDevice({
            productKey,
            deviceName,
            token,
            success:(res) => {
                this._currentStatus = {
                    status: 'success',
                    extraData: res,
                };
                this._notifyListeners();
            },
            fail:(err) => {
                this._currentStatus = {
                    status: 'fail',
                    extraData: {
                        code: 'bind',
                        msg: err
                    },
                };
                this._notifyListeners();
            },
        });
    }

    /**
     * 触发监听器
     */
    private _notifyListeners() {
        this._listenerManager.map((listener) => {
            listener(this._currentStatus);
        });
    }
};

export default new AddDevice();