import { DeviceList } from './index';
import {
    DeviceListChangeMessage
} from '@alife/tg-iot-interface-api-device-list';

declare let my: any;

/**
 * 设备列表刷新间隔 ms
 */
const SYNC_INTERVAL = 3000;
export class DeviceListAliPay extends DeviceList {
    private inSync: boolean = false;
    private syncDeviceListenerList = new Array<Function>();
    /**
     * 每次拉取设备列表都通知的列表
     */
    private deviceListenerList = new Array<Function>();

    constructor() {
        super()
    }

    public onDeviceListSync(listener: (info: Array<DeviceListChangeMessage>) => void) {
        if (this.deviceListenerList.indexOf(listener) != -1) {
            this.log("duplicate sync call.");
        } else {
            this.deviceListenerList.push(listener);
        }

        this.onDeviceListChange(listener)
    }

    public onDeviceListChange(listener: (info: Array<DeviceListChangeMessage>) => void) {
        if (this.syncDeviceListenerList.indexOf(listener) != -1) {
            this.log("duplicate sync call.");
        } else {
            this.syncDeviceListenerList.push(listener);
        }

        if (this.inSync) {
            return;
        }
        this.log("start sync")
        this.inSync = true;

        this.syncDeviceInfoTask();
    }

    private syncDeviceInfoTask() {
        if (!this.inSync) {
            return;
        }

        const oldDeviceList = this.deviceCache;
        const _this = this;

        this.getDeviceList({
            appInfo: this.localAppInfo,
            success: () => {
                this.notifyDeviceListUpdate(_this, oldDeviceList);
            },
            complete: () => {
                setTimeout(() => {
                    this.log("next loop")
                    this.syncDeviceInfoTask()
                }, SYNC_INTERVAL)
            }
        })
    }

    private notifyDeviceListUpdate(_this, oldDeviceList) {
        this.deviceListenerList.forEach(listener => {
            try { listener(_this.deviceCache); } catch(err) { console.error(err); }
        })

        let oldArr = {}, newArr = {};
        oldArr = Object.assign(oldArr, oldDeviceList);
        newArr = Object.assign(newArr, _this.deviceCache);

        let changedArr: any = [];
        Object.keys(oldArr).forEach(key => {
            if (newArr[key]) {
                delete newArr[key];
            } else {
                changedArr.push({
                    "devId": key,
                    "action": "delete"
                })
            }
        })

        Object.keys(newArr).forEach(key => {
            changedArr.push({
                "devId": key,
                "action": "add"
            })
        })

        if (changedArr.length > 0) {
            this.syncDeviceListenerList.forEach(listener => {
                try { listener(changedArr); } catch(err) { console.error(err); }
            })
        }
    }

    public offDeviceListChange(listener: (info: Array<DeviceListChangeMessage>) => void) {
        const index = this.syncDeviceListenerList.indexOf(listener);
        if (index == -1) {
            this.log('invalid listener');
            return;
        }

        this.syncDeviceListenerList.splice(index, 1);

        const index2 = this.deviceListenerList.indexOf(listener);
        if (index2 > -1) {
            this.deviceListenerList.splice(index2, 1)
        }

        if (this.syncDeviceListenerList.length == 0) {
            this.inSync = false;
            this.log("stop sync")
        } else {
            this.log("listener unregister.")
        }
    }
}

export default new DeviceListAliPay()
