import deviceList, { DeviceList } from './index';
import {
    DeviceListChangeMessage
} from '@alife/tg-iot-interface-api-device-list';

declare let my: any;

export class DeviceListTG extends DeviceList {
    private listenerList = new Array<Function>();

    constructor() {
        super()
        my.on("onDeviceListChange", e => {
            const deviceCache = deviceList.deviceCache;
            deviceList.getDeviceList({
                appInfo: deviceList.localAppInfo,
                success: (devices) => {
                    this.notifyDeviceListUpdate(deviceCache)
                }
            })
        })
    }

    public onDeviceListChange(listener: (info:  Array<DeviceListChangeMessage>) => void) {
        if (this.listenerList.indexOf(listener) != -1) {
            console.warn('duplicated listener');
            return;
        }

        this.listenerList.push(listener);
        my.call("getService", { sdkName: "DeviceMonitor", serviceName: "BoneAccsService", minSDKVer: "*" }, service => {
            my.call("callService", { serviceId: service.serviceId, methodName: "subscribe", args: [{ "name": "device_list_change" }] }, result => {
                console.log(JSON.stringify(result))
                my.call("callContainer", { serviceName: "BoneEvent", args: [] }, (data => { }));
            })
        })
    }

    private notifyDeviceListUpdate(oldDeviceList) {
        let oldArr = {}, newArr = {};
        oldArr = Object.assign( oldArr,oldDeviceList);
        newArr = Object.assign( newArr,this.deviceCache);

        let changedArr: any = [];
        Object.keys(oldArr).forEach(key => {
            if (newArr[key]) {
                delete newArr[key];
            } else {
                changedArr.push({
                    "devId": key,
                    "action": "delete"
                })
            }
        })

        Object.keys(newArr).forEach(key => {
            changedArr.push({
                "devId": key,
                "action": "add"
            })
        })

        this.listenerList.forEach(listener => {
            listener(changedArr)
        })
    }

    public offDeviceListChange(listener: (info:  Array<DeviceListChangeMessage>) => void) {
        const index = this.listenerList.indexOf(listener);
        if (index == -1) {
            console.warn('invalid listener');
            return;
        }

        this.listenerList.splice(index, 1);

        if (this.listenerList.length == 0) {
            my.call("getService", { sdkName: "DeviceMonitor", serviceName: "BoneAccsService", minSDKVer: "*" }, service => {
                my.call("callService", { serviceId: service.serviceId, methodName: "unsubscribe", args: [{ "name": "device_list_change" }] }, result => { })
            })
        }
    }
}

export default new DeviceListTG();
