import mtop from '../mtop/index.alipay';
import IDeviceListAPI, {
    Device,
    GetDeviceOptions,
    DeviceListChangeMessage
} from '@alife/tg-iot-interface-api-device-list';

const API_NAME = 'mtop.alibaba.aicloud.iot.getAllDevicesForAlipay';
const API_VERSION = '1.0';
const PARAMS = {};
const DEVICE_INFO = '';

export class DeviceCache {
    [key: string]: Device;
}

/**
 * 设备列表刷新间隔 ms
 */
const SYNC_INTERVAL = 3000;

export class DeviceList implements IDeviceListAPI {

    private appInfoDefault = { "appName": "天猫精灵", "appVersion": "4.8.0", "deviceModel": "Pixel2", "osVersion": "google/walleye/walleye: 10/QP1A.191005.007.A1/5908163: user/release-keys", "platform": "Android" };
    public localAppInfo = this.appInfoDefault;

    private _deviceCache: DeviceCache;

    constructor() {
        this._deviceCache = new DeviceCache();
    }

    get deviceCache(): DeviceCache {
        return this._deviceCache;
    }

    public getDeviceList(request: GetDeviceOptions) {
        const appInfo = request.appInfo;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        const self = this;

        if (JSON.stringify(appInfo) != JSON.stringify(this.localAppInfo)) {
            this.localAppInfo = appInfo;
        }

        mtop.sendMtop({
            api: API_NAME,
            v: API_VERSION,
            needLogin: true,
            data: {
                params: JSON.stringify(PARAMS),
                deviceInfo: DEVICE_INFO,
                appInfo: JSON.stringify(appInfo),
            },
            success: (ret: any) => {
                self._handleResponse(ret, success);
            },
            fail,
            complete,
        });
    };

    private _handleResponse(ret: any, success: Function) {
        // console.log('>> _handleResponse', JSON.stringify(ret));

        const { devices, zones } = ret;

        this._deviceCache = new DeviceCache();

        devices.map(device => {
            const dev = device as Device;
            this._deviceCache[dev.devId.toLowerCase()] = dev;
        });

        success(devices);
    };

    public onDeviceListChange(callback: (msg: Array<DeviceListChangeMessage>) => void) {

    }

    public offDeviceListChange(callback: (msg: Array<DeviceListChangeMessage>) => void) {

    }

    public log(msg) {
        console.log(msg)
    }
}

export default new DeviceList();
