import { TMPSDK } from './index';
import deviceList from '../device-list/index.alipay';
import {
    DeviceStatusChangeMessage
} from '@alife/tg-iot-interface-api-device-panel'

export class TMPSDKAlipay extends TMPSDK {
    private syncListenerList = new Array<Function>();
    private oldDeviceListCache = deviceList.deviceCache;

    private deviceListChangeListener = (info) => {
        // this.log(JSON.stringify(info));
        this.notifyDeviceChange(this.oldDeviceListCache, deviceList.deviceCache);
    }

    public onDeviceStatusChange(listener: (mesageArray: Array<DeviceStatusChangeMessage>) => void) {
        if (this.syncListenerList.indexOf(listener) != -1) {
            this.log("duplicate sync call.");
        } else {
            this.syncListenerList.push(listener);
            this.oldDeviceListCache = deviceList.deviceCache;
            deviceList.onDeviceListSync(this.deviceListChangeListener)
        }
    }

    private notifyDeviceChange(originArr, currentArr) {

        let SyncDeviceInfoArr: any = [];
        Object.keys(currentArr).forEach(key => {
            const currentDevice = currentArr[key];
            const originDevice = originArr[key];

            if (currentDevice.screenModelFunctionParams && currentDevice.screenModelFunctionParams.status) {
                if (!originDevice || !originDevice.screenModelFunctionParams) {
                    this.log("origin device is null")
                } else if (currentDevice.screenModelFunctionParams.status != originDevice.screenModelFunctionParams.status) {
                    SyncDeviceInfoArr.push({
                        action: "status_update",
                        attr: JSON.parse(currentDevice.screenModelFunctionParams.status),
                        deviceId: key
                    });
                } else {
                    this.log("same")
                }
            } else {
                this.log("不是iot设备");
            }
        })

        if (SyncDeviceInfoArr.length > 0) {
            this.syncListenerList.forEach(listener => {
                listener(SyncDeviceInfoArr);
            })
        }

        this.oldDeviceListCache = currentArr;

    }

    private isEquals(jsonStirng1, jsonString2) {
        //todo 精确比较
        return jsonStirng1 === jsonString2;
    }

    public offDeviceStatusChange(listener: (mesageArray: Array<DeviceStatusChangeMessage>) => void) {
        const index = this.syncListenerList.indexOf(listener);
        if (index == -1) {
            this.log('invalid listener');
            return;
        }

        this.syncListenerList.splice(index, 1);

        if (this.syncListenerList.length == 0) {
            deviceList.offDeviceListChange(this.deviceListChangeListener)
        } else {
            this.log("listener unregister.")
        }
    }
};

export default new TMPSDKAlipay();