import IDevicePanelAPI, {
    DeviceControlOptions,
    GetDeviceStatusOptions,
    GetProductInfoOptions,
    GetProductTSLOptions,
    DeleteDeviceOptions,
    DeviceStatusChangeMessage,
    EMTHOD_ATTRIBUTE_SET,
    EMTHOD_SERVICE_INVOKE,
    MeshDeviceControlOptions,
} from '@alife/tg-iot-interface-api-device-panel';

const SDK_NAME = 'BoneTmpSDK';
const SERVICE_NAME = 'BoneTmpService';

declare let my: any;

export class TMPSDKTG implements IDevicePanelAPI {

    private serviceId;

    private listenerList = new Array<Function>();

    constructor() {
        /**
         * 预取 serviceId
         */
        this.getServiceIdAsync(serviceId=>{
            this.serviceId = serviceId;
        });

        /**
         * 监听设备变化状态
         */
        my.on("iot.syncDeviceInfo", e => {
            this.listenerList.forEach(listener => {
                listener(e);
            });
        });
    }

    public controlDevice(request: DeviceControlOptions){
        const deviceId = request.deviceId;
        const method = request.method;
        const params = request.params;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        const skillId = request.skillId;

        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }

        if (!method
            || (EMTHOD_ATTRIBUTE_SET !== method && EMTHOD_SERVICE_INVOKE !== method)) {
            throw new Error('method is invalid');
        }

        if (!params) {
            throw new Error('params is invalid');
        }

        this.getServiceIdAsync( serviceId => {
            try {
                my.call('callService', {
                    serviceId,
                    methodName: 'controlDevice',
                    args: [
                        deviceId,
                        skillId,
                        method,
                        params,
                    ],
                }, (result) => {
                    success(result);
                    complete && complete(result);
                });
            } catch(err) {
                fail && fail(err);
                complete && complete(err);
            }
        });
    };

    /**
     * 蓝牙 Mesh 设备专属控制接口
     * <p/>
     * 一方接口，仅限内部使用
     * @param request 请求描述
     */
    public controlMeshDevice(request: MeshDeviceControlOptions){
        const deviceId = request.deviceId;
        const params = request.params;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        const skillId = request.skillId;

        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }

        if (!params) {
            throw new Error('params is invalid');
        }

        this.getServiceIdAsync( serviceId => {
            try {
                my.call('callService', {
                    serviceId,
                    methodName: 'controlMeshDevice',
                    args: [
                        deviceId,
                        skillId,
                        params,
                    ],
                }, (result) => {
                    success(result);
                    complete && complete(result);
                });
            } catch(err) {
                fail && fail(err);
                complete && complete(err);
            }
        });
    };

    public getDeviceStatus(request: GetDeviceStatusOptions){
        const deviceId = request.deviceId;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        const skillId = request.skillId;

        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }

        this.getServiceIdAsync( serviceId => {
            try {
                my.call('callService', {
                    serviceId,
                    methodName: 'getDeviceStatus',
                    args: [
                        deviceId,
                        skillId,
                    ],
                }, (result) => {
                    success(result);
                    complete && complete(result);
                });
            } catch(err) {
                fail && fail(err);
                complete && complete(err);
            }
        });
    };

    public getProductInfo(request: GetProductInfoOptions){
        const productKey = request.productKey;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;

        if (!productKey) {
            throw new Error('productKey is invalid');
        }

        this.getServiceIdAsync( serviceId => {
            try {
                my.call('callService', {
                    serviceId,
                    methodName: 'getProductInfo',
                    args: [
                        productKey,
                    ],
                }, (result) => {
                    success(result);
                    complete && complete(result);
                });
            } catch(err) {
                fail && fail(err);
                complete && complete(err);
            }
        });
    };

    public getProductTSL(request: GetProductTSLOptions){
        const deviceId = request.deviceId;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        const skillId = request.skillId;

        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }

        this.getServiceIdAsync( serviceId => {
            try {
                my.call('callService', {
                    serviceId,
                    methodName: 'getProductTSL',
                    args: [
                        deviceId,
                        skillId,
                    ],
                }, (result) => {
                    success(result);
                    complete && complete(result);
                });
            } catch(err) {
                fail && fail(err);
                complete && complete(err);
            }
        });
    };

    public onDeviceStatusChange(listener: (mesageArray: Array<DeviceStatusChangeMessage>) => void) {
        if (this.listenerList.indexOf(listener) != -1) {
            console.warn('duplicated listener');
            return;
        }

        this.listenerList.push(listener);
    }

    public offDeviceStatusChange(listener: (mesageArray: Array<DeviceStatusChangeMessage>) => void) {
        const index = this.listenerList.indexOf(listener);
        if (index == -1) {
            console.warn('invalid listener');
            return;
        }

        this.listenerList.splice(index, 1);
    }

    public deleteDevice(request: DeleteDeviceOptions){
        const deviceId = request.deviceId;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        const skillId = request.skillId;

        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }

        this.getServiceIdAsync( serviceId => {
            try {
                my.call('callService', {
                    serviceId,
                    methodName: 'deleteDevice',
                    args: [
                        deviceId,
                        skillId,
                    ],
                }, (result) => {
                    success(result);
                    complete && complete(result);
                });
            } catch(err) {
                fail && fail(err);
                complete && complete(err);
            }
        });
    };

    public getDeviceConnectStatus(request: GetDeviceStatusOptions){
        const deviceId = request.deviceId;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        const skillId = request.skillId;

        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }

        this.getServiceIdAsync( serviceId => {
            try {
                my.call('callService', {
                    serviceId,
                    methodName: 'getDeviceConnectStatus',
                    args: [
                        deviceId,
                        skillId,
                    ],
                }, (result) => {
                    success(result);
                    complete && complete(result);
                });
            } catch(err) {
                fail && fail(err);
                complete && complete(err);
            }
        });
    };

    /**
     * 获取需要的 serviceId
     * @param callback 获得到 serviceId 后的回调
     */
    private getServiceIdAsync(callback: Function){
        if (this.serviceId) {
            callback && callback(this.serviceId);
        } else {
            my.call(
                "getService",
                { sdkName: SDK_NAME, serviceName: SERVICE_NAME, minSDKVer: "*" },
                ( service ) => { callback && callback(service.serviceId); }
            );
        }
    }
};

export default new TMPSDKTG();