import IMTopAPI, { MTopOptions } from '@alife/tg-iot-interface-api-mtop';
import { APIError } from '@alife/tg-iot-interface-api-common';

declare const my:any;

const defaultRequest: MTopOptions = {
    api: '',
    v:'1.0',
    data: {},
    needLogin: false,
    success: (ret) => {
        console.warn('unhandled response:', JSON.stringify(ret));
    }
};

export class MTop implements IMTopAPI {
    private _debug: boolean = true;

    public setDebug(debug:boolean) {
        this._debug = debug;
    }

    public sendMtop(request: MTopOptions) {
        // 添加默认值
        request = Object.assign({}, defaultRequest, request);

        const api:string = request.api;
        const v:string = request.v;
        const needLogin: boolean = request.needLogin || false;
        let data:object = request.data;
        const success:Function = request.success;
        const fail = request.fail;
        const complete = request.complete;

        if (!api) {
            throw new Error('api is invalid');
        }

        if (!v) {
            throw new Error('v is invalid');
        }

        try{
            if (this._debug) {
                console.log('>> before send mtop', api, v, needLogin, data);
            }

            // CC 上调用
            request.get(api, data).then((res) => {
                if (this._debug) {
                    console.log('>> after send mtop', JSON.stringify(res));
                }

                res.data = res.data || {}
                res.ret = res.ret || []
                if (res.retType === 0 || res['ret'][0] === 'SUCCESS::调用成功' || res.data.statusCode === '200' || res.data.success === 'true') {
                    success(res.data.model);
                    complete && complete(res.data.model);
                } else {
                    const responseError : APIError = {
                        code: res.data.msgCode,
                        msg: res.data.msgInfo,
                    }

                    // 从 ret 中获取 errorCode
                    if (!responseError.code) {
                        const retMessage = res.ret[0];
                        const messageArr = retMessage.split('::');
                        Object.assign(responseError, {
                            code: messageArr[0],
                            msg: messageArr[1]
                        });
                    }

                    console.warn('>> response error happens when send mtop', JSON.stringify(responseError));
                    fail && fail(responseError);
                    complete && complete(responseError);
                }
            }).catch(err => {
                console.warn('>> exception happens when send mtop', JSON.stringify(err));
                fail && fail(err);
                complete && complete(err);
            });
        } catch(err) {
            console.warn('>> exception happens when send mtop', JSON.stringify(err));
            fail && fail(err);
            complete && complete(err);
        }
    }
}

export default new MTop();