import { MtopApi } from './API'
import NativeApi from './nativeApi'
// import { request } from '@ali/a8s';
// import '../depends/tg-iot-implements-api/lib/index.js'

export default class API{
  // mtop 接口分发
  distributeMtop() {
    this.baseData = {}
    this.deviceStatusCallback = () => {}
    this.attributes = {}
    if (this.Params && this.Params.type === 'sence') {
      this.Params = {}
      this.normalApi() // mtop接口分发
    } else {
      this.CCApi() // CC 上这些 mtop 接口分发
      if (!this.Params.isTest) {
        this.getAllDevices = NativeApi.getAllDevices
        this.getDeviceStatus = NativeApi.getDeviceStatus
        this.getDeviceInfo = NativeApi.getProductTSL
        this.container !== 'CC' && (this.deviceControl = NativeApi.controlDevice)
        this.getProductInfo = NativeApi.getProductInfo
        
        if ((this.systemInfo.app === 'TGenie' || this.systemInfo.app === 'alipay') && this.container !== 'CC') {
          // 支付宝小程序上用这个
          this.getDeviceFunctionCorpus = this.getDeviceFunctionCorpusForAlipay
          this.getMultiPortRouter = this.getMultiPortRouterForAlipay
          this.getDeviceDetail = this.getDeviceDetailForAlipay
        } else {
          // cc上用这个
          this.getDeviceFunctionCorpus = this.getDeviceFunctionCorpusForCC
          this.getAllChildDevsByParentDevId = this.getAllChildDevsByParentDevIdForCC
          this.getDeviceDetail = this.getDeviceDetailForCC
          this.getCorpusAndControlParamByDev = this.getCorpusAndControlParamByDevForCC
        }
      }
    }
  }

  // 设置参数
  setParams({ devId, productKey, skillId = 3404, deviceName = ''}) {
    this.Params = this.Params || { deviceInfo: {} }
    this.Params.deviceInfo = this.Params.deviceInfo || {}
    
    this.Params.deviceInfo.devId = devId || this.Params.deviceInfo.devId
    this.Params.deviceInfo.productKey = productKey || this.Params.deviceInfo.productKey
    this.Params.deviceInfo.skillId = skillId || this.Params.deviceInfo.skillId
    this.Params.deviceInfo.deviceName = deviceName || this.Params.deviceInfo.deviceName
  }

  async CCApi() {
    MtopApi.forEach((item) => {
      const [method, apiName] = item.split('::')
      this[method] = (params = {}) => {

        // 注入默认参数 和 传入参数
        const defaultParams = this.mtopDistributeParams(method)
        const opt = {
          apiName,
          data: Object.assign({}, defaultParams, params)
        }
        return new Promise((resolve, reject) => {
          const checkResult = this.mtopCheckParms(method, Object.assign({}, defaultParams, params))
          if (checkResult.error) {
            return reject(`${method}接口：${checkResult.error}`)
          }

          if (this.systemInfo.app === 'GenieWebApps' || this.container === 'CC') {
            // CC 上调用
            let reqParams = {
              ignore_aliauth_autologin: "YES",
              apiName: opt.apiName,
              apiVersion: '1.0',
              needEcodeSign: true,
              useGenieEnv: true,
              data: opt.data
            };
            my.call('mtop', reqParams, function (res) {
              res.data = res.data || {};
              res.ret = res.ret || [];
              if (res.retType === 0 || res['ret'][0] === 'SUCCESS::调用成功' || res.data.statusCode === '200' || res.data.success === 'true') {
                resolve(res.data);
              } else {
                res.errorMessage = res.msgInfo;
                res.errorCode = res.msgCode;
                reject(res);
              }
            }, function (err) {
              err.errorMessage = err.msgInfo;
              err.errorCode = err.msgCode;
              reject(err);
            });
          } else {
            my.aiot.sendMtop({
              api: opt.apiName,
              data: opt.data,
              needLogin: true,
              success: (res) => {
                resolve({
                  model: res
                })
              },
              fail: err => {
                this.console && console.log('调用接口失败了：', JSON.stringify(err))
                err.errorMessage = err.msgInfo
                err.errorCode = err.msgCode
                reject(err)
              }
            });
          }
        })
      }
    })
  }

  // 获取基础信息
  getbaseInfo() {
    const deviceName = this.Params.deviceInfo && this.Params.deviceInfo.deviceName
    return new Promise((resolve, reject) => {
      this.baseData = {}
      if (deviceName) {
        this.baseData.deviceDetail = {
          deviceName: deviceName 
        }
      }

      // 10秒中超时
      this.baseInfoTimeout = setTimeout(() => {
        resolve({ error: '接口调用超时' })
      }, 20000)

      

      !deviceName && this.getDeviceDetail().then(deviceDetail => {
        this.console && console.log('获取设备详情返回时间点：', JSON.stringify(deviceDetail), JSON.stringify(new Date()))
        if (!deviceDetail.model.devId) {
          resolve({ msgInfo: '你没有该设备' })
          return
        }
        if (JSON.stringify(deviceDetail.model) !== '{}') {
          deviceDetail.model.deviceName = deviceDetail.model.alias || deviceDetail.model.devName || deviceDetail.model.devType
          this.baseData.deviceDetail = deviceDetail.model
          // 保存埋点数据
          this.saveSpmInfo(deviceDetail.model, (user_role) => {
            console.error('user_role', user_role)
            if (this.baseData.deviceInfo && this.baseData.productInfo) resolve(this.baseData)
          })

          if (deviceDetail.model.platform.name === 'SIGMESH' && !deviceDetail.model.shareInfo.isShareDevice) {
            // 本地控制逻辑
            console.error('本地控制逻辑>>>>')
            this.localControl(deviceDetail.model)
          }

          // 获取物的模型
          this.getDeviceInfo().then(deviceInfo => {
            deviceInfo.model = deviceInfo.model || {}
            this.console && console.log('获取物的模型返回时间点：', JSON.stringify(deviceInfo), JSON.stringify(new Date()))
            if (JSON.stringify(deviceInfo.model.tsl) !== '{}') {
              this.baseData.deviceInfo = deviceInfo.model
              if (this.baseData.productInfo && this.baseData.deviceDetail) resolve(this.baseData)
            } else {
              resolve({ msgInfo: '未获取到该设备的物模型信息' })
            }
          }).catch(err => {
            this.console && console.log('获取物模型失败：', JSON.stringify(err))
            resolve({ msgInfo: '获取物模型失败：' + (err.msg || err.msgInfo) })
          })

          // 获取设备信息
          this.getProductInfo().then(productInfo => {
            this.console && console.log('获取产品信息返回时间点：', JSON.stringify(productInfo), JSON.stringify(new Date()))
            if (JSON.stringify(productInfo.model) !== '{}') {
              this.baseData.productInfo = productInfo.model
              if (this.baseData.deviceInfo && this.baseData.deviceDetail) resolve(this.baseData)
            } else {
              resolve({
                msgInfo: '未获取到该设备的产品信息'
              })
            }
          }).catch(err => {
            this.console && console.log('获取产品信息失败：', JSON.stringify(err))
            resolve({ msgInfo: '获取产品信息失败：' + (err.msg || err.msgInfo) })
          })
        } else {
          resolve({ msgInfo: '未获取到设备详情信息' })
        }
      }).catch(err => {
        this.console && console.log('获取设备详情失败：', JSON.stringify(err))
        resolve({ msgInfo: '获取设备详情失败：' + (err.msg || err.msgInfo) })
      })
    })
  }

  // 保存埋点数据
  saveSpmInfo(deviceDetail = {}, callback = () => {}) {
    // 埋点
    const {devId, productKey, devTypeEn, platform = {}, skillId, sourceType, userId, shareInfo = {}, screenName = ''} = deviceDetail

    let user_role = 'owner'
    if (sourceType === 'family') {
      user_role = 'member'
    } else if (shareInfo.isShareDevice) {
      user_role = 'sharer'
    }
    const { app, model } = my.getSystemInfoSync()
    let container_type = ''
    if (app === 'UNKNOWN' && model === 'AliOSThings') { container_type = 'box_CC_MINI' }
    else if (app === 'genie' || app === 'GenieWebApps' || this.container === 'CC') { container_type = 'box_CC' }
    else if (app === 'TGenie') { container_type = 'app' } else { container_type = 'alipay' }
    
    this.spmParamsData = {
      spmA: this.spmA,
      spmB: this.spmB,
      container_type,
      platform_type: 'IoT',
      iot_type: platform.name,
      dev_type_en: devTypeEn,
      dev_id: devId,
      skill_id: skillId,
      user_id: userId,
      product_key: productKey,
      is_share: shareInfo.isShareDevice,
      user_role,// owner,member,sharer
      panel_type: screenName || this.Params.screenName || '宜控'
    }
    callback(user_role)
  }

  // mtop接口分发
  normalApi () {
    MtopApi.forEach((item) => {
      const [method, apiName] = item.split('::')
      this[method] = (params = {}) => {

        // 注入默认参数 和 传入参数
        const defaultParams = this.mtopDistributeParams(method)
        const opt = {
          apiName,
          data: Object.assign({}, defaultParams, params)
        }
        return new Promise((resolve, reject) => {
          const checkResult = this.mtopCheckParms(method, Object.assign({}, defaultParams, params))
          if (checkResult.error) {
            return reject(`${method}接口：${checkResult.error}`)
          }
          this.Mtop(opt).then((resp) => {
            resolve(resp)
          }, (error) => {
            reject(error)
          })
        })
      }
    })
  }

  // 注入默认参数 和 传入参数
  mtopDistributeParams (method) {
    const { devId, productKey, skillId } = this.Params.deviceInfo || {}
    switch (method) {
      case 'getDeviceStatus': return { devId, skillId }
      case 'deviceControl': return { devId, skillId }
      case 'getDeviceInfo': return { devId, skillId, productKey }
      case 'getDeviceDetail': return { devId, skillId }
      case 'getDeviceDetailForCC': return { devId, skillId }
      case 'getDeviceDetailForAlipay': return { devId, skillId }
      case 'getCorpusAndControlParamByDev': return { devId }
      case 'getCorpusAndControlParamByDevForCC': return { devId }
      case 'getProductInfo': return { productKey }
      case 'getDeviceFunctionCorpus': return { params: JSON.stringify({devId, skillId, productKey}) }
      case 'getDeviceFunctionCorpusForCC': return { params: JSON.stringify({devId, skillId, productKey}) }
      case 'getDeviceFunctionCorpusForAlipay': return { params: JSON.stringify({devId, skillId, productKey}) }
      case 'getAllDevices': return { appInfo: JSON.stringify({
        "osVersion": "13.5.1",
        "appName": "天猫精灵",
        "deviceModel": "iPhone12,   8",
        "appVersion": "4.6.0.7",
        "platform": "iOS"
        }), params: JSON.stringify({"terminalName":"天猫精灵手机","modelType":0}) 
      }
      default: return {}
    }
  }

  /**
   * motp 接口必传参数校验
   * @param {*} type 参数类型
   */
  mtopCheckParms (method, params) {
    let mustParams = {}
    switch (method) {
      case 'getDeviceDetail': mustParams = { devId: 'string', skillId: 'string' }
        break
    }

    for (let i in mustParams) {
      if (!params[i]) {
        return { error: `缺少必填参数${i}` }
      }
    }
    return { success: 200 }
  }
  // 是否可以控制
  couldIControl() {
    if (!this.meshDeviceStatus) return {}
    if (this.meshDeviceStatus.hasOnlineVoiceBox) return {}
    let msgInfo = ''
    console.log(this.meshDeviceStatus.AndroidLocationStatus, 'this.meshDeviceStatus.AndroidLocationStatus>>>>')
    if (!this.meshDeviceStatus.phoneMeshStatus) {
      msgInfo = '您的蓝牙开关还没打开哦'
    // } else if (!this.meshDeviceStatus.AndroidLocationStatus && this.isAndriod()) {
    //   msgInfo = '该设备需要打开定位权限才能控制哦'
    }
    return {
      msgInfo
    }
  }

  // 本地控制逻辑
  localControl(device) {
    let { devId, productKey, skillId } = this.Params.deviceInfo || {}

    // 默认的蓝牙状态
    this.meshDeviceStatus = {
      isMeshDevice: device.platform.name, // 是否是蓝牙设备
      meshConnestStatus: null, // 蓝牙连接设备的状态
      phoneMeshStatus: null, // 手机打开蓝牙的状态
      hasOnlineVoiceBox: null, // 是否有音箱在线的状态
      // meshConnecting: null, // 蓝牙正在连接中...
      AndroidLocationStatus: null, // 蓝牙定位开启状态
      AndroidLocationPermissionStatus: null, // 蓝牙定位权限状态
    }

    // 开始连接蓝牙
    this.startLocalConnect()

    my.aiot.getDeviceConnectStatus({
      deviceId: devId,
      productKey,
      skillId: Number(skillId),
      success: (res) => {
        this.meshDeviceStatus.meshConnestStatus = res.connectByPhone // 蓝牙连接设备的状态
        this.meshDeviceStatus.hasOnlineVoiceBox = res.connectBySundbox // 是否有音箱在线的状态
        this.meshDeviceStatus.phoneMeshStatus = res.ext.bleEnable // 更新蓝牙打开状态
        this.meshDeviceStatus.AndroidLocationStatus = res.ext.locationEnable // 更新全局定位连接设备的状态
        
        console.error('设备连接log>>>>', res)
      },
      fail: (err) => {
        console.error('设备连接log失败>>>>', err)
      }
    });

    // 兼听音箱状态
    my.aiot.onGenieBoxStatusChange((event) => {
      console.log('事件监听-音箱在线状态>>>>', event);
      this.meshDeviceStatus.hasOnlineVoiceBox = event.isOnline
    });

    // 兼听设备状态
    my.aiot.onConnectStatusChange((event) => {
      console.error('事件监听-设备在线状>>>>', event);
      let status = event.connectStatus === 0 || event.status === 0
      if (event.connectStatus === 0 || event.status === 0) {
        status = true
        this.meshDeviceStatus.phoneMeshStatus = true // 更新蓝牙打开状态
      } else {
        status = false
        // this.shouldConnecting = true
      }
      if (event.status === 2) {
        // this.meshDeviceStatus.phoneMeshStatus = false // 更新蓝牙打开状态
        // 开始连接蓝牙
        this.startLocalConnect()
      } else {
        this.meshDeviceStatus.phoneMeshStatus = true // 更新蓝牙打开状态
      }
      this.meshDeviceStatus.meshConnestStatus = status // 蓝牙连接设备的状态
    });
  }
  // 开始连接蓝牙
  startLocalConnect() {
    let { devId, productKey, skillId } = this.Params.deviceInfo || {}
    console.log('连接蓝牙>>>>')
    // 开始连接
    my.aiot.startLocalConnect({ 
      deviceId: devId,
      productKey,
      skillId: Number(skillId),
      success: (res) => {
        this.meshDeviceStatus.meshConnestStatus = res.connectStatus === 0 // 蓝牙连接设备的状态
        console.error('startLocalConnect成功>>>>', res)
      },
      fail: (err) => {
        console.error('startLocalConnect失败>>>>', err)
      }
    })
  }

  // 获取设备状态 并 兼听
  getDeviceStatusAndListening({ callback = () => {}, isList = false}, status) {
    let { devId, productKey, skillId } = this.Params.deviceInfo || {}
    this.waitReceiveDeviceStatus = false
    setTimeout(() => {
      this.deviceStatusCallback = callback // 设备属性返回
      
      // 小程序IDE上，轮询方法、持续获取设备状态
      console.log('第一次获取数据 devId', devId, isList)
      if (!isList && devId) {
        this.keepGetDeviceStatus()
      }
      

      // 轮询设备状态
      // this.pollingOpen = true
      // this.pollingDeviceStatus()

      if (!this.Params.isTest) {
        // 长链接接口数据
        console.log('打开长链接兼容new')
        my.aiot.offSyncDeviceInfo(this.listenDeviceInfo)
        this.listenDeviceInfo = (data) => {
          console.log(status, '收到的长链接数据', JSON.stringify(data))
          if (!data) return console.log('长链接收到data为空')
          for (let i = 0; i < data.length; i++) {
            if(status === 'allListen') {
              this.attributes = Object.assign(this.attributes, data[i].attr) // 保留当前最新的属性状态
              this.deviceStatusCallback({
                type: 'syncAllDeviceResult',
                devId: data[i].deviceId,
                attr: data[i].attr
              })
            } else if (data[i].deviceId === devId) {
              if (this.waitReceiveDeviceStatus) return console.log('拦截长链接的数据')// 控制的3秒内，不接收消息，3秒后，主动获取
              this.attributes = Object.assign(this.attributes, data[i].attr) // 保留当前最新的属性状态
              this.deviceStatusCallback({
                type: 'syncDeviceInfoResult',
                devId: data[i].deviceId,
                attr: data[i].attr
              })
            }
          }
          // [{"deviceId":"a80171ee000000667d6b07da78000000","attr":{"powerstate":0}}]
        }
        my.aiot.onSyncDeviceInfo(this.listenDeviceInfo)
      } else {
        // 轮询设备状态
        this.pollingDeviceStatus()
      }
    }, 200)
  }

  // 小程序IDE上，轮询方法、持续获取设备状态
  keepGetDeviceStatus() {
    const { productKey } = this.Params.deviceInfo || {}
    if (!productKey) return console.log('keepGetDeviceStatus 无PK不从重新轮询')// 无PK不从重新轮询
    // 主动获取设备状态
    console.log('获取设备状态中...')
    this.getDeviceStatus().then(resp => {
      if (this.waitReceiveDeviceStatus) return console.log('拦截数据')// 控制的3秒内，不接收消息，3秒后，主动获取
      if (resp.model && resp.model.attr) {
        this.deviceData = resp.model.attr
        this.attributes = Object.assign(this.attributes, resp.model.attr) // 保留当前最新的属性状态
        this.deviceStatusCallback({
          type: 'APIResult',
          attr: resp.model.attr
        })
      } else {
        this.stopPolling()
        this.deviceStatusCallback({
          type: 'errorResult',
          errorInfo: resp,
          attr: {}
        })
      }
    }).catch(err => {
      console.log('获取设备状态失败', err)
      this.stopPolling()
      this.deviceStatusCallback({
        type: 'errorResult',
        errorInfo: err,
        attr: {}
      })
    })
  }

  // 控制设备状态属性
  setDeviceStatus({attr, method, spm}, params = {}, groupControl) {
    

    const { devId, skillId } = this.Params.deviceInfo || {}
    const option = {
      params: attr
    }
    // params.method ||
    option.method = method || 'thing.attribute.set'
    option.devId = params.devId || devId
    option.msgId = ''
    option.version = '1.0'

    return new Promise((resolve, reject) => {
      const waitControlTime = 3000

      // 判断是否无法控制
      const controlInfo = this.couldIControl()
      console.log('controlInfo>>>>', controlInfo)
      if(controlInfo.msgInfo) {
        this.deviceStatusCallback({
          type: 'warning',
          errorInfo: controlInfo.msgInfo,
          attr: this.attributes
        })
        reject({ msgInfo: controlInfo.msgInfo })
        return
      }

      // 操作过快提示
      console.error('***', this.alreadyControl)
      if (this.alreadyControl) {
        // console.log(JSON.stringify(this.attributes), '&&&')
        // this.respCallback({ model: { attr: this.attributes } })
        this.deviceStatusCallback({
          type: 'warning',
          errorInfo: '请不要操作过快哦',
          attr: this.attributes
        })
        reject({ msgInfo: '请不要操作过快哦' })
        return
      }
      this.alreadyControl = true
      this.SET_TIMEOUT && clearTimeout(this.SET_TIMEOUT)
      this.SET_TIMEOUT = setTimeout(() => {
        this.alreadyControl = false
      }, 1000)

      // waitControlTime 时间内种内
      this.waitReceiveDeviceStatus = true
      // 停止轮询
      this.stopPolling()
      clearTimeout(this.controlTimeOut)
      

      // 立刻修改面板状态
      this.attributes = Object.assign(this.attributes, {...option.params})
      console.error('点击存的属性>>>>', this.attributes)
      this.deviceStatusCallback({
        type: 'setResult',
        // attr: {...this.deviceData, ...option.params},
        attr: {...option.params},
        params
      })

      // 设置设备属性
      console.log('params.devId || devId', params.devId || devId, spm)
      // 控制埋点
      if (spm !== 'fastControl') {
        this.setBaseSpm('控制属性点击', this.spmA + '.' + this.spmB + '.' + this.getControlStatus(attr), 'controlClickEvent', 'control', { control_property: this.getControlStatus(attr), control_path: 'cloud'}) // spm数据处理
        this.setPageSpmControlTimes() // 控制次数埋点
      } else {
        this.setPageSpmErlangFastControl() // 快捷控制次数埋点
      }
      this.deviceControl({
        params: JSON.stringify(option),
        devId: params.devId || devId,
        skillId: params.skillId || skillId
      }).then(resp => {
        // 控制轮询
        this.controlTimeOut = setTimeout(() => {
          this.waitReceiveDeviceStatus = false
          
          // 抓取设备状态
          this.keepGetDeviceStatus()
  
          // 继续轮询
          this.pollingOpen && this.pollingDeviceStatus() 
        }, waitControlTime)

        if (resp.model && resp.model !== 'null') {
          this.setBaseSpm('控制发送成功', this.spmA + '.' + this.spmB + '.' + this.getControlStatus(attr), 'exposureEvent', 'control_send', { control_property: this.getControlStatus(attr), control_path: 'cloud'}) // spm数据处理
          resolve(resp);
        } else {
          if (spm !== 'fastControl') {
            this.setBaseSpm('控制执行失败', this.spmA + '.' + this.spmB + '.' + this.getControlStatus(attr), 'exposureEvent', 'control_run_fail', { control_property: this.getControlStatus(attr), control_path: 'cloud'}) // spm数据处理
            this.setPageSpmControlFailTimes() // 操作失败次数埋点
          } else {
            this.setPageSpmErlangFastControlFail() // 快捷操作失败次数埋点
          }
          let msgInfo = '操作失败'
          // 音箱不在线,无法推送指令
          if (err.msg === '音箱不在线,无法推送指令' || err.message === '音箱不在线,无法推送指令' || err.msgInfo === '音箱不在线,无法推送指令' || err.kBoneJSBridgeErrorMessage === '音箱不在线,无法推送指令' || err.localizedMsg === '音箱不在线,无法推送指令') { 
            msgInfo = '音箱不在线,无法推送指令'
          } else if (err.msg === '请不要操作过快哦' || err.message === '请不要操作过快哦' || err.msgInfo === '请不要操作过快哦' || err.kBoneJSBridgeErrorMessage === '请不要操作过快哦' || err.localizedMsg === '请不要操作过快哦') { 
            msgInfo = '请不要操作过快哦'
          } else if (err.msg === '定位权限没有开启' || err.message === '定位权限没有开启' || err.msgInfo === '定位权限没有开启' || err.kBoneJSBridgeErrorMessage === '定位权限没有开启' || err.localizedMsg === '定位权限没有开启') { 
            msgInfo = '定位权限没有开启'
          } else if (err.msg === '手机设置中的天猫精灵应用定位权限未开启' || err.message === '手机设置中的天猫精灵应用定位权限未开启' || err.msgInfo === '手机设置中的天猫精灵应用定位权限未开启' || err.kBoneJSBridgeErrorMessage === '手机设置中的天猫精灵应用定位权限未开启' || err.localizedMsg === '手机设置中的天猫精灵应用定位权限未开启') { 
            msgInfo = '手机设置中的天猫精灵应用定位权限未开启'
          }
          // 
          this.deviceStatusCallback({
            type: 'warning',
            errorInfo: msgInfo,
            attr: this.attributes
          })
          reject({ msgInfo });
        }
        // resolve(resp)
      }).catch(err => {
        if (err.msgInfo === 'Operation queue has unfinished tasks') {
          return
        }
        // 控制重新主动获取数据
        this.controlTimeOut = setTimeout(() => {
          this.waitReceiveDeviceStatus = false
          
          // 抓取设备状态
          this.keepGetDeviceStatus()
  
          // 继续轮询
          this.pollingOpen && this.pollingDeviceStatus() 
        }, waitControlTime)
        
        let msgInfo = '操作失败'
        // 音箱不在线,无法推送指令
        if (err.msg === '音箱不在线,无法推送指令' || err.message === '音箱不在线,无法推送指令' || err.msgInfo === '音箱不在线,无法推送指令' || err.kBoneJSBridgeErrorMessage === '音箱不在线,无法推送指令' || err.localizedMsg === '音箱不在线,无法推送指令') { 
          msgInfo = '音箱不在线,无法推送指令'
        } else if (err.msg === '请不要操作过快哦' || err.message === '请不要操作过快哦' || err.msgInfo === '请不要操作过快哦' || err.kBoneJSBridgeErrorMessage === '请不要操作过快哦' || err.localizedMsg === '请不要操作过快哦') { 
          msgInfo = '请不要操作过快哦'
        } else if (err.msg === '定位权限没有开启' || err.message === '定位权限没有开启' || err.msgInfo === '定位权限没有开启' || err.kBoneJSBridgeErrorMessage === '定位权限没有开启' || err.localizedMsg === '定位权限没有开启') { 
          msgInfo = '定位权限没有开启'
        } else if (err.msg === '手机设置中的天猫精灵应用定位权限未开启' || err.message === '手机设置中的天猫精灵应用定位权限未开启' || err.msgInfo === '手机设置中的天猫精灵应用定位权限未开启' || err.kBoneJSBridgeErrorMessage === '手机设置中的天猫精灵应用定位权限未开启' || err.localizedMsg === '手机设置中的天猫精灵应用定位权限未开启') { 
          msgInfo = '手机设置中的天猫精灵应用定位权限未开启'
        }
        // 前往设备详情页面
        err.msgInfo = msgInfo
        if (spm !== 'fastControl') {
          this.setBaseSpm('控制执行失败', this.spmA + '.' + this.spmB + '.' + this.getControlStatus(attr), 'exposureEvent', 'control_run_fail', { control_property: this.getControlStatus(attr), control_path: 'cloud'}) // spm数据处理
          this.setPageSpmControlFailTimes() // 操作失败次数埋点
        } else {
          this.setPageSpmErlangFastControlFail() // 快捷操作失败次数埋点
        }
        reject(err)
        
      })
    })
  }
  // 获取控制属性
  getControlStatus(attr) {
    let str = ''
    for (let i in attr) {
      str += i + '_'
    }
    return str.slice(0, str.length - 1)
  }

  // 轮询获取设备状态
  pollingDeviceStatus() {
    const { productKey } = this.Params.deviceInfo || {}
    if (!productKey) return console.log('pollingDeviceStatus 无PK不从重新轮询')// 无PK不从重新轮询
    clearTimeout(this.polling)
    this.polling = setTimeout(() => {
      // 持续获取设备状态
      this.keepGetDeviceStatus()

      // 继续轮询
      this.pollingDeviceStatus() 
    }, 3000)
  }

  // 停止轮询
  stopPolling() {
    clearTimeout(this.polling)
  }
  // 前往设备详情页面
  goDeviceDetail(callback = () => {}) {
    if (this.systemInfo.app === 'TGenie') {
      const { devId, skillId } = this.Params.deviceInfo || {}
      let  url =`https://market.m.taobao.com/app/genie/weex/pages/device-manager?wh_weex=true&from=control&_ali_nav_title_=设备详情&devId=${devId}&skillId=${skillId}`;
      my.call('algservices', {
        api:'common.pageRouter.openUrl',
        params: {
          url: url,
        }
      }, (result) => {
        console.info('openUrl ===>', JSON.stringify(result))
      });
    } else {
      callback()
    }
  }
}