import RESULT_CODE from './resultCode.js'
export default class JsBridge {
  // 调用 jsBridge
  jsBridge (api, params = {}, jsBridgeApi = 'algservices') {
    if (!api) return console.error('api不能不传')
    return new Promise((resolve, reject) => {
      my.call(jsBridgeApi, {
        api,
        params
      }, (resp) => {
        // 未填必填参数 - 报错返回
        if (resp.error && resp.errorMessage) {
          if (this.isIos()) {
            try {
              reject(JSON.parse(resp.errorMessage || '{}'))
            } catch (error) {
              reject({errorCode: 'FAIL', errorMessage: api + '接口调用失败'})
            }
            
          } else {
            reject({errorCode: resp.errorCode || 'FAIL', errorMessage: resp.errorMessage})
          }
        }

        // 参数错后 - 报错返回
        if (resp.result && resp.result.errorCode) {
          reject(resp.result)
        }

        if (resp.error) reject(resp)

        resolve(resp.result)
      }, (error) => {
        reject(error)
      })
    })
  }

  // Mtop 接口
  Mtop ({
    apiName,
    data = {},
    extensions = {}
  } = {}) {
    return new Promise((reslove, reject) => {
      if (!apiName) return reject({ error: 'apiName不能不传' })
      const apiData = {
        apiName,
        useGenieEnv: true,
        apiVersion: '1.0',
        data,
        ...extensions
      }
      if (apiName === 'mtop.alibaba.aicloud.scene.createOrUpdateScene') {
        apiData.method = 'POST'
      }
      this.jsBridge('common.network.alilabsMtop', apiData).then((resp) => {
        resp.success += ''
        resp.result && (resp.result.success += '')
        if (resp.error || (resp.success !== 'true' && (!resp.result || resp.result.success !== 'true'))) {
          if (resp.result && resp.result.msgInfo) {
            resp.result.errorMessage = resp.result.msgInfo
            resp.result.errorCode = resp.result.msgCode
            reject(resp.result)
          } else if (resp.msgInfo) {
            resp.errorMessage = resp.msgInfo;
            resp.errorCode = resp.msgCode;
            reject(resp);
          } else {
            reject(resp)
          }
          
        }
        resp = resp.result || resp
        try {
          if (resp.model !== undefined) {
            if (resp.model.retValue) {
              resp.result = resp.model.retValue
              delete resp.model
            } else {
              resp.result = resp.model
              delete resp.model
            }
          }
        } catch (error) {}
        reslove(resp)
      }).catch(error => {
        if (error.result && error.result.msgInfo) {
          error.result.errorMessage = error.result.msgInfo
          error.result.errorCode = error.result.msgCode
          reject(error.result)
        } else {
          reject(error)
        }
      })
    })
  }

  /**
   *  接口返回信息检查
   * @param {*} method 方法名
   * @param {*} data 数据
   */
  resultCheck (method, res) {
    if (res.error) {
      return res
    }

    if (!res || !res.data) {
      if (res.originValue) {
        return Object.assign(RESULT_CODE.NET_ERR, { msgInfo: res.originValue, msgCode: res.ret })
      }
      return Object.assign(RESULT_CODE.NET_ERR, { respond: res })
    }
    if (res.retType && res.retType !== 0) {
      const { ret } = res
      let ret_msgInfo = ''
      let ret_msgCode = ''
      if (ret[0].indexOf('::') > -1) {
        const msgArray = ret[0].split('::')
        ret_msgCode = msgArray[0]
        ret_msgInfo = msgArray[1]
      }
      return Object.assign(RESULT_CODE.NET_ERR, { msgInfo: ret_msgInfo, msgCode: ret_msgCode })
    }
    if (res.data.success !== 'true') {
      const { msgInfo, msgCode } = res.data
      return Object.assign(RESULT_CODE.NET_ERR, { msgInfo, msgCode })
    }
    
    if (res.data.success === 'true' && res.data.model) {
      const data = { model: res.data.model }
      if (res.data.extensions) {
        data.extensions = res.data.extensions
      }
      return Object.assign(RESULT_CODE.SUCCEED, data)
    }
    return res
  }
  /**
   * 不同方法检查分发
   * @param {*} method 方法名
   * @param {*} res 接口返回信息
   * @param {*} secondInfo 二次待检查信息
   */
  distributeCheck (method, res, secondInfo) {
    switch (method) {
      case 'getDeviceStatus': return this.getDeviceStatusResultCheck(res)
      case 'deviceControl': return this.deviceControlResultCheck(res, secondInfo)
      default: return 'NONE'
    }
  }
}