"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// import DeviceSmartConfig from '@alife/iot-mini-program-device-smart-config';
var prepare_1 = __importDefault(require("./prepare"));
var bind_1 = __importDefault(require("./bind"));
var AddDevice = /** @class */ (function () {
    function AddDevice() {
        this._listenerManager = new Array();
    }
    Object.defineProperty(AddDevice.prototype, "currentAddDeviceStatus", {
        get: function () {
            return this._currentStatus;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * 开始添加设备
     * @param options 开始添加设备的信息描述
     */
    AddDevice.prototype.startAddDevice = function (options) {
        var _this = this;
        var productKey = options.productKey, ssid = options.ssid, fail = options.fail, complete = options.complete;
        if (!productKey) {
            throw new Error('productKey is missing');
        }
        if (!ssid) {
            throw new Error('ssid is missing');
        }
        if (this._currentStatus && this._currentStatus.status === '')
            this._currentStatus = {
                status: 'prechecking',
            };
        this._notifyListeners();
        prepare_1.default.getDeviceConfigToken({
            productKey: productKey,
            success: function (res) {
                _this._doProvision(options, res);
            },
            fail: fail,
            complete: complete,
        });
    };
    /**
     * 停止添加设备
     * @param options 停止添加设备的信息描述
     */
    AddDevice.prototype.stopAddDevice = function (options) {
        // if (!this._deviceSmartConfigClient) {
        // return;
        // }
        // this._deviceSmartConfigClient.stop();
        // this._deviceSmartConfigClient = null;
    };
    /**
     * 开始监听设备添加过程变化
     * @param listener 监听器
     */
    AddDevice.prototype.onAddDeviceStatusChagne = function (listener) {
        if (this._listenerManager.indexOf(listener) !== -1) {
            console.warn('duplicated listener');
            return;
        }
        this._listenerManager.push(listener);
    };
    /**
     * 停止监听设备添加过程变化
     * @param listener 需要停止监听的监听器; 若为空，则停止所有监听器
     */
    AddDevice.prototype.offAddDeviceStatusChange = function (listener) {
        if (!listener) {
            this._listenerManager.splice(0, this._listenerManager.length);
            return;
        }
        var index = this._listenerManager.indexOf(listener);
        if (index === -1) {
            console.warn('invalid listener');
            return;
        }
        this._listenerManager.splice(index, 1);
    };
    /**
     * 进行配网
     * @param productKey 待配网的产品
     * @param token 产品的配网秘钥
     * @param success 成功回调
     * @param fail 失败回调
     * @param complte 完成回调
     */
    AddDevice.prototype._doProvision = function (options, token) {
        this._currentStatus = {
            status: 'provisioning',
        };
        this._notifyListeners();
        var productKey = options.productKey, ssid = options.ssid, pw = options.pw, timeout = options.timeout;
        console.log('>> _doProvision', productKey, ssid, pw, token, timeout);
        // this._deviceSmartConfigClient = new DeviceSmartConfig({
        //     productKey,
        //     securityKey: token,
        //     ssid,
        //     password:pw,
        //     timeout,
        //     onProgress:(options) => {
        //         console.log('>> onProgress', JSON.stringify(options));
        //         const { status, message } = options;
        //         if (status === 'fail') {
        //             this._currentStatus = {
        //                 status: 'fail',
        //                 extraData: {
        //                     code: 'timeout',
        //                     msg: message,
        //                 }
        //             }
        //             this._notifyListeners();
        //             this._deviceSmartConfigClient.stop();
        //             this._deviceSmartConfigClient = null;
        //             return;
        //         }
        //         if (status === 'success') {
        //             const {productKey, deviceName, token} = message;
        //             this._doBind(productKey, deviceName, token);
        //             this._deviceSmartConfigClient.stop();
        //             this._deviceSmartConfigClient = null;
        //             return;
        //         }
        //     }
        // });
        // this._deviceSmartConfigClient.start().then(()=>{
        //     console.log('>> _deviceSmartConfigClient.start', 'success');
        //     options.success({});
        // }).catch((err)=>{
        //     console.log('>> _deviceSmartConfigClient.start', 'failed', err);
        //     options.fail && options.fail(err);
        // });
    };
    /**
     * 绑定设备
     * @param productKey 待绑定的产品
     * @param deviceName 待绑定的设备名称
     * @param token 待绑定的 token
     */
    AddDevice.prototype._doBind = function (productKey, deviceName, token) {
        var _this = this;
        this._currentStatus = {
            status: 'binding',
            extraData: {
                productKey: productKey,
                deviceName: deviceName,
                token: token,
            },
        };
        this._notifyListeners();
        bind_1.default.bindDevice({
            productKey: productKey,
            deviceName: deviceName,
            token: token,
            success: function (res) {
                _this._currentStatus = {
                    status: 'success',
                    extraData: res,
                };
                _this._notifyListeners();
            },
            fail: function (err) {
                _this._currentStatus = {
                    status: 'fail',
                    extraData: {
                        code: 'bind',
                        msg: err
                    },
                };
                _this._notifyListeners();
            },
        });
    };
    /**
     * 触发监听器
     */
    AddDevice.prototype._notifyListeners = function () {
        var _this = this;
        this._listenerManager.map(function (listener) {
            listener(_this._currentStatus);
        });
    };
    return AddDevice;
}());
;
exports.default = new AddDevice();
