"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var tg_iot_interface_api_common_1 = require("@alife/tg-iot-interface-api-common");
var tg_iot_interface_api_device_panel_1 = require("@alife/tg-iot-interface-api-device-panel");
var index_rtos_1 = __importDefault(require("./mtop/index.rtos"));
var DeviceCache = /** @class */ (function () {
    function DeviceCache() {
    }
    return DeviceCache;
}());
exports.DeviceCache = DeviceCache;
;
var aiot = {
    _deviceCache: new DeviceCache(),
    sendMtop: index_rtos_1.default.sendMtop.bind(index_rtos_1.default),
};
// wrapp event "onDeviceInfoSync"
var deviceStatusChangelistenerList = new Array();
var onSyncDeviceInfo = function (listener) {
    if (deviceStatusChangelistenerList.indexOf(listener) != -1) {
        console.warn('duplicated listener');
        return;
    }
    deviceStatusChangelistenerList.push(listener);
};
var offSyncDeviceInfo = function (listener) {
    var index = deviceStatusChangelistenerList.indexOf(listener);
    if (index == -1) {
        console.warn('invalid listener');
        return;
    }
    deviceStatusChangelistenerList.splice(index, 1);
};
// wrapp event "onDeviceListChanged"
var deviceListChangelistenerList = new Array();
var onDeviceListChange = function (listener) {
    if (deviceListChangelistenerList.indexOf(listener) != -1) {
        console.warn('duplicated listener');
        return;
    }
    deviceListChangelistenerList.push(listener);
};
var offDeviceListChange = function (listener) {
    var index = deviceListChangelistenerList.indexOf(listener);
    if (index == -1) {
        console.warn('invalid listener');
        return;
    }
    deviceListChangelistenerList.splice(index, 1);
};
Object.assign(aiot, { onSyncDeviceInfo: onSyncDeviceInfo, offSyncDeviceInfo: offSyncDeviceInfo, onDeviceListChange: onDeviceListChange, offDeviceListChange: offDeviceListChange });
var API_LIST = [
    'getDeviceList',
    'controlDevice',
    'getDeviceStatus',
    'getProductInfo',
    'getProductTSL',
];
API_LIST.forEach(function (method) {
    aiot[method] = function (options) {
        var success = options.success, fail = options.fail, complete = options.complete, deviceId = options.deviceId, args = __rest(options, ["success", "fail", "complete", "deviceId"]);
        // 为以下接口补充 skillId
        if (method === 'controlDevice'
            || method === 'getDeviceStatus'
            || method === 'getProductTSL') {
            var lowerCaseDeviceId = deviceId.toLowerCase();
            if (aiot._deviceCache[lowerCaseDeviceId]) {
                var device = aiot._deviceCache[lowerCaseDeviceId];
                var skillId = device.extra.skillId;
                var productKey = device.productKey;
                Object.assign(args, {
                    skillId: skillId,
                    productKey: productKey,
                    devId: deviceId,
                });
            }
        }
        // 为控制接口拼接参数
        if (method === 'controlDevice') {
            var skillId = args.skillId, controlMethod = args.method, params = args.params;
            args = {
                devId: deviceId,
                skillId: skillId,
                params: JSON.stringify({
                    method: controlMethod || tg_iot_interface_api_device_panel_1.EMTHOD_ATTRIBUTE_SET,
                    params: params,
                    devId: deviceId,
                    "msgId": "",
                    "version": "1.0",
                }),
            };
        }
        try {
            my.call("aiot." + method, args, function (res) {
                var data = res.data;
                if (!data) {
                    var apiError = {
                        code: tg_iot_interface_api_common_1.ERROR_CODE,
                        msg: '无效的返回值'
                    };
                    fail && fail(apiError);
                    complete && complete(apiError);
                    return;
                }
                var isSuccess = data.success, model = data.model, msgCode = data.msgCode, msgInfo = data.msgInfo;
                if (isSuccess) {
                    if (method === 'getDeviceList') {
                        var devices = model.devices, zones = model.zones;
                        aiot._deviceCache = new DeviceCache();
                        devices.map(function (device) {
                            var dev = device;
                            aiot._deviceCache[dev.devId.toLowerCase()] = dev;
                        });
                    }
                    success(model);
                    complete && complete(model);
                }
                else {
                    var err = {
                        code: msgCode,
                        msg: msgInfo,
                    };
                    fail && fail(err);
                    complete && complete(res);
                }
            });
        }
        catch (err) {
            var apiError = {
                code: tg_iot_interface_api_common_1.ERROR_CODE,
                msg: JSON.stringify(err),
            };
            fail && fail(apiError);
            complete && complete(apiError);
        }
    };
});
var steupEventListener = function () {
    my.on('iot.onDeviceListChanged', function (msg) {
        console.log('>> iot.onDeviceListChanged for rtos', JSON.stringify(msg));
        var data = msg.data;
        if (!data) {
            console.warn('invalid data for message: iot.onDeviceListChanged');
            return;
        }
        // transfor map to array
        // like {"0":{"deviceId":"a80171ee000000667d6b07da78000000","powerstate":0}}}
        var DeviceListChangeMessageArray = Object.values(data);
        if (!DeviceListChangeMessageArray) {
            console.warn('invalid DeviceListChangeMessageArray');
            return;
        }
        deviceListChangelistenerList.map(function (listener) {
            listener(DeviceListChangeMessageArray);
        });
    });
    my.on('iot.syncDeviceInfo', function (msg) {
        console.info('>> iot.syncDeviceInfo', JSON.stringify(msg));
        var data = msg.data;
        if (!data) {
            console.warn('invalid data for message: iot.onDeviceInfoSync');
            return;
        }
        // transfor map to array
        // like {"0":{"deviceId":"a80171ee000000667d6b07da78000000","powerstate":0}}}
        var syncDeviceInfoArray = Object.values(data);
        var formatedAttrArray = new Array();
        syncDeviceInfoArray.map(function (item) {
            var deviceId = item.deviceId, attrs = __rest(item, ["deviceId"]);
            formatedAttrArray.push({
                deviceId: deviceId,
                attr: attrs,
            });
        });
        if (!formatedAttrArray) {
            console.warn('invalid formatedAttrArray');
            return;
        }
        deviceStatusChangelistenerList.map(function (listener) {
            listener(formatedAttrArray);
        });
    });
};
exports.steupEventListenerForRTOS = steupEventListener;
exports.default = aiot;
