import { APIOptions, APIError, ERROR_CODE } from "@alife/tg-iot-interface-api-common"
import { EMTHOD_ATTRIBUTE_SET, DeviceStatusChangeMessage } from '@alife/tg-iot-interface-api-device-panel'
import mtop from './mtop/index.cc';

declare let my: any;

const aiot = {
  sendMtop: mtop.sendMtop.bind(mtop),
};

// wrapp event "onSyncDeviceInfo"
const listenerList = new Array<Function>();
const onSyncDeviceInfo = (listener:Function)=>{
  if(listenerList.indexOf(listener) != -1){
    console.warn('duplicated listener');
    return;
  }

  listenerList.push(listener);
}

const offSyncDeviceInfo = (listener:Function)=>{
  const index = listenerList.indexOf(listener);
  if(index == -1){
    console.warn('invalid listener');
    return;
  }

  listenerList.splice(index, 1);
}

interface SyncDeviceInfo {
  name:string,
  type:string,
  data:{
    data: Array<DeviceStatusChangeMessage>,
  }
}

Object.assign(aiot, { onSyncDeviceInfo, offSyncDeviceInfo });

const API_LIST: Array<string> = [
  'controlDevice',
  'getDeviceStatus',
  'getProductInfo',
  'getProductTSL',
  'deletedevice'
];

API_LIST.forEach(method => {
  aiot[method] = (options: APIOptions) => {
    const { success, fail, complete, ...args } = options;

    // 控制设备时，默认 method 为设置属性
    if (method === 'controlDevice') {
      const { method: controlMethod } = args;

      if(!controlMethod){
        Object.assign(args, {
          method: EMTHOD_ATTRIBUTE_SET
        });
      }
    }

    try {
      my.call(`aiot_${method}`, args, (res: any) => {
        const { success:isSuccess, model, msgCode, msgInfo } = res;
        if(isSuccess) {
          const res = JSON.parse(model);

          success(res);
          complete && complete(res);
        } else {
          const err = {
            code: msgCode,
            msg: msgInfo,
          } as APIError;

          fail && fail(err);
          complete && complete(res);
        }
      });
    } catch (err) {
      const apiError = {
        code: ERROR_CODE,
        msg: JSON.stringify(err),
      } as APIError;
      fail && fail(apiError);
      complete && complete(apiError);
    }
  }
});

const setupEventListener = () => {
  my.on('iot.syncDeviceInfo', (msg) => {
    console.log('>> iot.onDeviceListChanged for cc', JSON.stringify(msg));
    const syncDeviceInfo = msg as SyncDeviceInfo;
    const { data } = syncDeviceInfo;

    if (!data) {
      console.warn('invalid DeviceStatusChangeMessage data');
      return;
    }

    const { data: arrayData } = data;

    if (!arrayData) {
      console.warn('invalid DeviceStatusChangeMessage array');
      return;
    }

    listenerList.forEach(listener => {
      listener(arrayData);
    });
  });
}

export const steupEventListenerForCC = setupEventListener;

export default aiot;