import IMTopAPI, { MTopOptions } from '@alife/tg-iot-interface-api-mtop';
import { APIError } from '@alife/tg-iot-interface-api-common';

const defaultRequest: MTopOptions = {
    api: '',
    v:'1.0',
    data: {},
    needLogin: false,
    success: (ret) => {
        console.warn('unhandled response:', JSON.stringify(ret));
    }
};

declare const my:any;

export class MTop implements IMTopAPI {

    private _defaultHeaders: any = {};
    private _debug: boolean = true;

    public setEnv(env:string) {
        if (!env) {
            // '' 代表线上
            env = '';
        }

        Object.assign(this._defaultHeaders, {
            'EagleEye-UserData':`scm_project=${env}`,
        });
    }

    public setDebug(debug:boolean) {
        this._debug = debug;
    }

    public sendMtop(request: MTopOptions) {
        // 添加默认值
        request = Object.assign({}, defaultRequest, request);

        const api:string = request.api;
        const v:string = request.v;
        const needLogin: boolean = request.needLogin || false;
        let data:object = request.data;
        const success:Function = request.success;
        const fail = request.fail;
        const complete = request.complete;

        if (!api) {
            throw new Error('api is invalid');
        }

        if (!v) {
            throw new Error('v is invalid');
        }

        // my.sendMtop 接口存在 bug,
        // 发送需要登录的 MTop 请求时，登录态处理会出错，使用 mtop 接口代替
        // my.sendMtop({
        //     api,
        //     v,
        //     needLogin,
        //     data,
        //     success,
        //     fail,
        //     complete,
        // });

        try{

            if (this._debug) {
                console.log('>> before send mtop', api, v, needLogin, data);
            }

            my.call('mtop', {
                apiName: api,
                apiVersion: v,   
                needEcodeSign: needLogin,
                useGenieEnv: needLogin,
                headers: this._defaultHeaders,
                data
            }, (ret) => {
                if (this._debug) {
                    console.log('>> after send mtop', JSON.stringify(ret));
                }

                const { data, ret:internalRet, api } = ret;

                if (!data) {
                    const invalidResponseError : APIError = {
                        code: 'invalid response',
                        msg: '无效的返回数据'
                    };
                    fail && fail(invalidResponseError);
                    complete && complete(invalidResponseError);
                    return;
                }

                // 标准的 MTOP 处理逻辑
                const { success:isSuccess, model, retValue, msgInfo, msgCode } = data;
                // isSuccess maybe true, false, 'true', 'false'
                // mtop get from def MTop Trigger use retValue instead of model
                if (isSuccess == 'true' || isSuccess === true) {
                    success(model || retValue);
                    complete && complete(model || retValue);
                } else {
                    const responseError : APIError = {
                        code: msgCode,
                        msg: msgInfo
                    };

                    // 从 ret 中获取 errorCode
                    if (!responseError.code) {
                        const retMessage = internalRet[0];
                        const messageArr = retMessage.split('::');
                        Object.assign(responseError, {
                            code: messageArr[0],
                            msg: messageArr[1]
                        });
                    }

                    console.warn('>> response error happens when send mtop', JSON.stringify(responseError));
                    fail && fail(responseError);
                    complete && complete(responseError);
                };
            });
        } catch(err) {
            console.warn('>> exception happens when send mtop', JSON.stringify(err));
            fail && fail(err);
            complete && complete(err);
        }
    }
}

export default new MTop();